<?php

/**
 * Skripta postavlja izbornike u blok administracija za nastavnike
 *
 * @package    local_dabar
 * @copyright  2019 Srce, Centar za e-učenje (http://www.srce.unizg.hr/ceu)
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

function local_dabar_extend_settings_navigation($nav, $context) {
	global $DB;

	// Poveznica na izvoz sigurnosne kopije u Dabar
	$coursenode = $nav->get('courseadmin', $nav::TYPE_COURSE);
	if($coursenode){
		$vu = false;
		$course = $DB->get_record('course', array('id'=>$context->instanceid));
		$category = core_course_category::get($course->category);
		$categories = explode('/', $category->path);
		foreach($categories as $category){
			if(get_config('local_dabar', "{$category}_api_url")){
				$vu = $category;
				break;
			}
		}
		
		if(!$course || !$coursenode || !has_capability('moodle/course:update', context_course::instance($course->id))) {
			return;
		}

		if($vu){
			$url = new moodle_url('/local/dabar/upload/select.php', array('courseid'=>$context->instanceid));
			$coursenode->add(get_string('export_to_dabar', 'local_dabar'), $url, $nav::TYPE_SETTING, null, 'dabar_select', new pix_icon('i/mnethost', '', 'moodle', array('class'=>'fa-upload')));
		}
	}
}

function get_progress_bar($currentstage) {
	global $PAGE;
	
	$stage = 6;
	$items = array();
	while ($stage > 0) {
		$classes = array('backup_stage');
		if (($stage - 1) == $currentstage) {
			$classes[] = 'backup_stage_next';
		} else if ($stage == $currentstage) {
			$classes[] = 'backup_stage_current';
		} else if ($stage < $currentstage) {
			$classes[] = 'backup_stage_complete';
		}
		$item = array('text' => $stage.'. '.get_string('currentstage'.$stage, 'local_dabar'), 'class' => join(' ', $classes));
		array_unshift($items, $item);
		$stage--;
	}
	return progress_bar($items);
}

function progress_bar(array $items) {
	foreach ($items as &$item) {
		$text = $item['text'];
		unset($item['text']);
		if (array_key_exists('link', $item)) {
			$link = $item['link'];
			unset($item['link']);
			$item = html_writer::link($link, $text, $item);
		} else {
			$item = html_writer::tag('span', $text, $item);
		}
	}
	return html_writer::tag('div', join(get_separator(), $items), array('class' => 'backup_progress clearfix'));
}

function get_backup_information($tempdir) {
	global $CFG;
	
	$info = new stdclass(); // Final information goes here
	
	$backuptempdir = make_backup_temp_directory('', false);
	$moodlefile = $backuptempdir . '/' . $tempdir . '/moodle_backup.xml';
	$coursefile = $backuptempdir . '/' . $tempdir . '/course/course.xml';
	if (!file_exists($moodlefile)) { // Shouldn't happen ever, but...
		throw new backup_helper_exception('missing_moodle_backup_xml_file', $moodlefile);
	}
	
	// Load the entire file to in-memory array
	$xmlparser = new progressive_parser();
	$xmlparser->set_file($moodlefile);
	$xmlprocessor = new restore_moodlexml_parser_processor();
	$xmlparser->set_processor($xmlprocessor);
	$xmlparser->process();
	$infoarr = $xmlprocessor->get_all_chunks();
	if (count($infoarr) !== 1) { // Shouldn't happen ever, but...
		throw new backup_helper_exception('problem_parsing_moodle_backup_xml_file');
	}
	$infoarr = $infoarr[0]['tags']; // for commodity
	
	if (file_exists($coursefile)) {
		$xmlparsercourse = new progressive_parser();
		$xmlparsercourse->set_file($coursefile);
		$xmlprocessorcourse = new restore_moodlexml_parser_processor();
		$xmlprocessorcourse->add_path('/course');
		$xmlparsercourse->set_processor($xmlprocessorcourse);
		$xmlparsercourse->process();
		$infoarrcourse = $xmlprocessorcourse->get_all_chunks();
		$infoarr['course_extra'] = $infoarrcourse[0]['tags'];
	}
	
	// Let's build info
	$info->backup_release = $infoarr['backup_release'];
	$info->backup_date    = $infoarr['backup_date'];
	$info->original_wwwroot         = $infoarr['original_wwwroot'];
	$info->type   =  $infoarr['details']['detail'][0]['type'];
	// Some initially empty containers
	$info->activities = array();
	$info->lang = map_languguages($infoarr['course_extra']['lang']);
	
	// Now the contents
	$contentsarr = $infoarr['contents'];
	if (isset($contentsarr['course']) && isset($contentsarr['course'][0])) {
		$info->course = new stdclass();
		$info->course = (object)$contentsarr['course'][0];
		$info->course->settings = array();
		$info->course->summary = strip_tags($infoarr['course_extra']['summary']);
		$info->course->fullname = $infoarr['course_extra']['fullname'];
	}
	if (isset($contentsarr['activities']) && isset($contentsarr['activities']['activity'])) {
		$activityarr = $contentsarr['activities']['activity'];
		foreach ($activityarr as $activity) {
			$activity = (object)$activity;
			$activity->settings = array();
			$activities[] = $activity;
		}
		$info->activities = $activities;
	}
	
	$info->root_settings = array(); // For root settings
	// Now the settings, putting each one under its owner
	$settingsarr = $infoarr['settings']['setting'];
	foreach($settingsarr as $setting) {
		switch ($setting['level']) {
			case 'root':
				$info->root_settings[$setting['name']] = $setting['value'];
				break;
		}
	}

	return $info;
}

function map_languguages($language){
	$lang_list = array("en"=>"eng","fr"=>"fre","hr"=>"hrv","de"=>"ger","es"=>"spa","it"=>"ita");
	if(!$lang_list[$language]){
		return 'hrv';
	}
	else{
		return $lang_list[$language];
	}
}

function human_filesize($bytes, $decimals = 2) {
	$sz = 'BKMGTP';
	$factor = floor((strlen($bytes) - 1) / 3);
	return sprintf("%.{$decimals}f", $bytes / pow(1024, $factor)) . @$sz[$factor];
}

?>