<?php

require_once('../../../config.php');
require_once($CFG->dirroot . '/backup/util/includes/restore_includes.php');
require_once('../bagit/bagit.php');
require_once('../restClient.php');

raise_memory_limit(MEMORY_EXTRA);
$courseid = required_param('courseid', PARAM_INT);
$pathnamehash = required_param('pathnamehash', PARAM_ALPHANUM);
$contenthash = required_param('contenthash', PARAM_ALPHANUM);
$stage    = required_param('stage', PARAM_INT);
$location = optional_param('location', null, PARAM_URL);

$context = context_course::instance($courseid);
list($context, $course, $cm) = get_context_info_array($context->id);

if($_POST['tocourse']){
	$urltogo = $CFG->wwwroot.'/course/view.php?id='.$course->id;
	redirect($urltogo);
	die;
}

$category = core_course_category::get($course->category);
$categories = explode('/', $category->path);
foreach($categories as $category){
	if(get_config('local_dabar', "{$category}_api_url")){
		$vu = $category;
		break;
	}
}

$PAGE->set_url(new moodle_url('/local/dabar/upload/select.php', array('courseid'=>$course->id)));
$PAGE->set_context($context);
$PAGE->set_pagelayout('admin');
$PAGE->add_body_class('path-backup');

$params = array();
$PAGE->requires->jquery();
$PAGE->requires->jquery_plugin('ui');
$PAGE->requires->jquery_plugin('ui-css');

require_login($course->id);
require_capability('local/dabar:api_upload', $context);

$SESSION->post_data = array_merge($SESSION->post_data, $_POST);

$PAGE->set_title($course->shortname . ': ' . get_string('currentstage'.$stage, 'local_dabar'));
$PAGE->set_heading($course->fullname);
echo $OUTPUT->header();

echo html_writer::start_tag('div', array('class' => 'backup-restore'));
echo get_progress_bar($stage);

if(!$location){
	// Prepare Moodle backup file
	$fs = get_file_storage();
	$storedfile = $fs->get_file_by_hash($pathnamehash);
	$backupfile = str_replace('.mbz', '', $storedfile->get_filename());
	$backuptempdir = make_backup_temp_directory($backupfile, false);
	if($storedfile->copy_content_to($backuptempdir.'/'.$storedfile->get_filename())){
		$mbz = $backuptempdir.'/'.$storedfile->get_filename();
	}
	else{
		throw new moodle_exception('error');
	}
	
	//Prepare XML files
	$xml_backup = new SimpleXMLElement('<mods/>');
	$access = $xml_backup->addChild('accessCondition', $SESSION->post_data['file']['access_rights']);
	$access->addAttribute('type', 'restriction on access');
	$terms = $xml_backup->addChild('accessCondition', $SESSION->post_data['file']['terms_of_use']);
	$terms->addAttribute('type', 'use and reproduction');
	if($SESSION->post_data['file']['access_rights'] == 'embargoedAccess'){
		$originInfo = $xml_backup->addChild('originInfo');
		$date = "{$SESSION->post_data['file']['embargo']['year']}-{$SESSION->post_data['file']['embargo']['month']}-{$SESSION->post_data['file']['embargo']['day']}";
		$dateOther = $originInfo->addChild('dateOther', $date);
		$dateOther->addAttribute('type', 'embargoEnd');
	}
	$xml_backup_file = $backuptempdir.'/file_metadata.xml';
	file_put_contents($xml_backup_file, $xml_backup->asXML());
	
	$xml_mods = new SimpleXMLElement('<?xml version="1.0" encoding="UTF-8"?><modsCollection/>');
	$xml_mods->addAttribute('xmlns', 'http://www.loc.gov/mods/v3', null);
	$xml_mods->addAttribute('xmlns:dabar', 'http://dabar.srce.hr/standards/schema/1.0', 'xmlns');
	$xml_mods->addAttribute('xmlns:mods', 'http://www.loc.gov/mods/v3', 'xmlns');
	$mods = $xml_mods->addChild('mods');
	$mods->addAttribute('ID', 'master');
	foreach($SESSION->post_data['language'] as $lang){
		$language = $mods->addChild('language');
		$languageTerm = $language->addChild('languageTerm', $lang['language']);
		$languageTerm->addAttribute('type', 'code');
		$languageTerm->addAttribute('authority', 'iso639-2b');
	}
	foreach($SESSION->post_data['title'] as $tit){
		$titleInfo = $mods->addChild('titleInfo');
		$titleInfo->addAttribute('lang', $tit['language']);
		$title = $titleInfo->addChild('title', $tit['title']);
	}
	foreach($SESSION->post_data['abstract'] as $abs){
		$abstract = $mods->addChild('abstract', $abs['abstract']);
		$abstract->addAttribute('lang', $abs['language']);
	}
	foreach($SESSION->post_data['keyword'] as $key){
		$subject = $mods->addChild('subject');
		$subject->addAttribute('lang', $key['language']);
		$topic = $subject->addChild('topic', $key['keyword']);
	}
	foreach($SESSION->post_data['discipline'] as $disc){
		$subject = $mods->addChild('subject');
		$subject->addAttribute('authority', 'nvzz.hr');
		$disciplines = json_decode(file_get_contents('../json/disciplines.json'));
		$topic = $subject->addChild('topic', $disc['area']);
		$topic = $subject->addChild('topic', $disciplines->{$disc['area']}->name->en);
		$topic->addAttribute('lang', 'eng');
		$topic = $subject->addChild('topic', $disc['field']);
		$topic = $subject->addChild('topic', $disciplines->{$disc['area']}->children->{$disc['field']}->name->en);
		$topic->addAttribute('lang', 'eng');
		if($disciplines->{$disc['area']}->children->{$disc['field']}->children->{$disc['branch']}->name->en){
			$topic = $subject->addChild('topic', $disc['branch']);
			$topic = $subject->addChild('topic', $disciplines->{$disc['area']}->children->{$disc['field']}->children->{$disc['branch']}->name->en);
			$topic->addAttribute('lang', 'eng');
		}
	}
	$part = $mods->addChild('part');
	$detail = $part->addChild('detail');
	$detail->addAttribute('type', 'version');
	$number = $detail->addChild('number', $SESSION->post_data['version']['version']);
	$extension = $mods->addChild('extension');
	$verzijaRada = $extension->addChild('dabar:verzijaRada', $SESSION->post_data['status']['status'], 'http://dabar.srce.hr/standards/schema/1.0');
	$verzijaRada->addAttribute('authority', 'HRZVO-KR-VerzijaRada');
	foreach($SESSION->post_data['requirements'] as $req){
		$TehnickiZahtjev = $extension->addChild('dabar:TehnickiZahtjev', null, 'http://dabar.srce.hr/standards/schema/1.0');
		$VrstaTehnickogZahtjeva = $TehnickiZahtjev->addChild('dabar:VrstaTehnickogZahtjeva', $req['type'], 'http://dabar.srce.hr/standards/schema/1.0');
		$VrstaTehnickogZahtjeva->addAttribute('authority', 'HRZVO-KR-VrstaTehnickogZahtjeva');
		$Naziv = $TehnickiZahtjev->addChild('dabar:Naziv', $req['name'], 'http://dabar.srce.hr/standards/schema/1.0');
		$NajmanjaVerzija = $TehnickiZahtjev->addChild('dabar:NajmanjaVerzija', $req['min_version'], 'http://dabar.srce.hr/standards/schema/1.0');
		$NajvecaVerzija = $TehnickiZahtjev->addChild('dabar:NajvecaVerzija', $req['max_version'], 'http://dabar.srce.hr/standards/schema/1.0');
	}
	$locationURL = $mods->addChild('location');
	$url = $locationURL->addChild('url', $SESSION->post_data['url']['url']);
	$url->addAttribute('displayLabel', 'URL');
	$Trajanje = $extension->addChild('dabar:Trajanje', 
			(($SESSION->post_data['duration']['learning_time']['year']) ? $SESSION->post_data['duration']['learning_time']['year'] : '00').'-'.
			(($SESSION->post_data['duration']['learning_time']['year']) ? $SESSION->post_data['duration']['learning_time']['month'] : '00').'-'.
			(($SESSION->post_data['duration']['learning_time']['year']) ? $SESSION->post_data['duration']['learning_time']['day'] : '00').'-'.
			(($SESSION->post_data['duration']['learning_time']['year']) ? $SESSION->post_data['duration']['learning_time']['hour'] : '00').'-'.
			(($SESSION->post_data['duration']['learning_time']['year']) ? $SESSION->post_data['duration']['learning_time']['minute'] : '00')
	, 'http://dabar.srce.hr/standards/schema/1.0');
	$OstaliTehnickiZahtjevi = $extension->addChild('dabar:OstaliTehnickiZahtjevi', $SESSION->post_data['other_requirements']['other_requirements'], 'http://dabar.srce.hr/standards/schema/1.0');
	$kontaktZaCjelovitiTekst = $extension->addChild('dabar:kontaktZaCjelovitiTekst', $SESSION->post_data['contact']['email'], 'http://dabar.srce.hr/standards/schema/1.0');
	foreach($SESSION->post_data['author'] as $aut){
		$name = $mods->addChild('name');
		$name->addAttribute('type', 'personal');
		$role = $name->addChild('role');
		$roleTerm = $role->addChild('roleTerm', 'Author');
		$roleTerm->addAttribute('type', 'text');
		$roleTerm->addAttribute('authority', 'marcrelator');
		$namePart = $name->addChild('namePart', $aut['firstname']);
		$namePart->addAttribute('type', 'given');
		$namePart = $name->addChild('namePart', $aut['lastname']);
		$namePart->addAttribute('type', 'family');
		$role = $name->addChild('role');
		$roleTerm = $role->addChild('roleTerm', $aut['role']);
		$roleTerm->addAttribute('type', 'text');
		$roleTerm->addAttribute('authority', 'HRZVO-KR-Publikacija_Osoba');
		foreach($aut['identifier'] as $ident){
			$nameIdentifier = $name->addChild('nameIdentifier', $ident['identifier']);
			$nameIdentifier->addAttribute('type', $ident['type']);
		}
		$nameIdentifier = $name->addChild('nameIdentifier', $aut['email']);
		$nameIdentifier->addAttribute('type', 'email');
		foreach($aut['affiliation'] as $aff){
			$affiliation = $name->addChild('affiliation', $aff['affiliation']);
			$affiliation->addAttribute('lang', $aff['language']);
		}
	}
	foreach($SESSION->post_data['participation'] as $part){
		$originInfo = $mods->addChild('originInfo');
		$originInfo->addAttribute('eventType', 'creation');
		if($part['from']['year'] && $part['from']['month'] && $part['from']['day']){
			$date = "{$part['from']['year']}-{$part['from']['month']}-{$part['from']['day']}";
		}
		elseif($part['from']['year'] && $part['from']['month']){
			$date = "{$part['from']['year']}-{$part['from']['month']}";
		}
		else{
			$date = "{$part['from']['year']}";
		}
		$dateOther = $originInfo->addChild('dateOther', $date);
		$dateOther->addAttribute('point', 'start');
		$dateOther->addAttribute('encoding', 'iso8601');
		if($part['to']['year'] && $part['to']['month'] && $part['to']['day']){
			$date = "{$part['to']['year']}-{$part['to']['month']}-{$part['to']['day']}";
		}
		elseif($part['to']['year'] && $part['to']['month']){
			$date = "{$part['to']['year']}-{$part['to']['month']}";
		}
		else{
			$date = "{$part['to']['year']}";
		}
		$dateOther = $originInfo->addChild('dateOther', $date);
		$dateOther->addAttribute('point', 'end');
		$dateOther->addAttribute('encoding', 'iso8601');
	}
	$genre = $mods->addChild('genre', 'info:eu-repo/semantics/learningObject');
	$genre->addAttribute('authorityURI', 'http://purl.org/REP/standards/info-eu-repo');
	$genre = $mods->addChild('genre', 'obrazovni sadržaj');
	$genre->addAttribute('lang', 'hrv');
	$genre = $mods->addChild('genre', 'educational content');
	$genre->addAttribute('lang', 'eng');
	foreach($SESSION->post_data['educational'] as $edu){
		$relatedItem = $mods->addChild('relatedItem');
		$relatedItem->addAttribute('type', 'constituent');
		$relatedItem->addAttribute('displayLabel', 'Obrazovni kontekst');
		$lrtype = json_decode(file_get_contents('../json/lrtype.json'));
		$genre = $relatedItem->addChild('genre', $edu['learning_resource_type']);
		$genre->addAttribute('authority', 'HRZVO-KR-VrstaObrazovnogSadrzaja');
		$genre->addAttribute('lang', 'hrv');
		$genre->addAttribute('valueURI', 'HRZVO-KR-VrstaObrazovnogSadrzaja:'.$lrtype->{$edu['learning_resource_type']}->num);
		$genre = $relatedItem->addChild('genre', $lrtype->{$edu['learning_resource_type']}->en);
		$genre->addAttribute('authority', 'HRZVO-KR-VrstaObrazovnogSadrzaja');
		$genre->addAttribute('lang', 'eng');
		$genre->addAttribute('valueURI', 'HRZVO-KR-VrstaObrazovnogSadrzaja:'.$lrtype->{$edu['learning_resource_type']}->num);
		$genre = $relatedItem->addChild('genre', 'info:eu-repo/semantics/other');
		$genre->addAttribute('authorityURI', 'http://purl.org/REP/standards/info-eu-repo');
		$targetAudience = $relatedItem->addChild('targetAudience', $edu['end_user_role']);
		$targetAudience->addAttribute('authority', 'HRZVO-KR-CiljanaSkupinaKorisnika');
		$subject = $relatedItem->addChild('subject');
		$subject->addAttribute('authority', 'HRZVO-KR-ObrazovniKontekst');
		$occupation = $subject->addChild('occupation', $edu['context']);
		$occupation->addAttribute('lang', 'hrv');
		$obkontekst = json_decode(file_get_contents('../json/context.json'));
		$occupation = $subject->addChild('occupation', $obkontekst->{$edu['context']}->en);
		$occupation->addAttribute('lang', 'en');
		$targetAudience = $relatedItem->addChild('targetAudience', (($edu['age_range']['from']) ? $edu['age_range']['from'] : '00').'-'.(($edu['age_range']['to']) ? $edu['age_range']['to'] : '00'));
		$targetAudience->addAttribute('displayLabel', 'Dob korisnika');
		$subject = $relatedItem->addChild('subject');
		$subject->addAttribute('authority', 'HRVO-KR-TezinaObrazovnogSadrzaja');
		$topic = $subject->addChild('topic', $edu['difficulty']);
		$topic->addAttribute('lang', 'hrv');
		$tezina = json_decode(file_get_contents('../json/difficulty.json'));
		$topic = $subject->addChild('topic', $tezina->{$edu['difficulty']}->en);
		$topic->addAttribute('lang', 'en');
		$date = (($edu['learning_time']['year']) ? $edu['learning_time']['year'] : '00').'-'.
				(($edu['learning_time']['year']) ? $edu['learning_time']['month'] : '00').'-'.
				(($edu['learning_time']['year']) ? $edu['learning_time']['day'] : '00').'-'.
				(($edu['learning_time']['year']) ? $edu['learning_time']['hour'] : '00').'-'.
				(($edu['learning_time']['year']) ? $edu['learning_time']['minute'] : '00');
		if($date != '00-00-00-00-00'){
			$subject = $relatedItem->addChild('subject');
			$temporal = $subject->addChild('temporal', $date);
		}
		$abstract = $relatedItem->addChild('abstract', $edu['abstract']);
		$abstract->addAttribute('type', 'opis');
	}
	$note = $mods->addChild('note', $SESSION->post_data['note']['note']);
	$note->addAttribute('type', 'public');
	$note = $mods->addChild('note', $SESSION->post_data['admin_note']['admin_note']);
	$note->addAttribute('type', 'internal');
	$typeOfResource = $mods->addChild('typeOfResource', $SESSION->post_data['fileformat']['fileformat']);
	$accessCondition = $mods->addChild('accessCondition', $SESSION->post_data['file']['terms_of_use']);
	$accessCondition->addAttribute('type', 'use and reproduction');
	$accessCondition = $mods->addChild('accessCondition', $SESSION->post_data['file']['access_rights']);
	$accessCondition->addAttribute('type', 'restriction on access');
	if($SESSION->post_data['file']['access_rights'] == 'embargoedAccess'){
		$originInfo = $mods->addChild('originInfo');
		$date = "{$SESSION->post_data['file']['embargo']['year']}-{$SESSION->post_data['file']['embargo']['month']}-{$SESSION->post_data['file']['embargo']['day']}";
		$dateOther = $originInfo->addChild('dateOther', $date);
		$dateOther->addAttribute('type', 'embargoEnd');
	}
	$recordInfo = $mods->addChild('recordInfo');
	$recordContentSource = $recordInfo->addChild('recordContentSource', $course->id);
	$recordContentSource->addAttribute('authority', 'source');
	$recordContentSource = $recordInfo->addChild('recordContentSource', parse_url(get_config('local_dabar', "{$vu}_api_url"), PHP_URL_HOST));
	$xml_mods_file = $backuptempdir.'/mods.xml';
	$content = str_replace('xmlns:xmlns="xmlns" ', '', $xml_mods->asXML());
	file_put_contents($xml_mods_file, $content);
	
	define('BASE_DIR', $backuptempdir.'/'.$backupfile);
	$baginfo = array(
		'UREDNIK_IME' => $USER->firstname,
		'UREDNIK_PREZIME' => $USER->lastname,
		'UREDNIK_OIB' => ($USER->profile['oib']) ? $USER->profile['oib'] : '0000000000',
		'OBJEKT_AKTIVAN' => '0',
		'Payload-Oxum' => (filesize($mbz)+filesize($xml_backup_file)+filesize($xml_mods_file)).".3",
		'Bagging-Date' => date('Y-m-d'),
		'Bag-Size' => human_filesize((filesize($mbz)+filesize($xml_backup_file)+filesize($xml_mods_file)), 1)
	);
	$bag = new BagIt(BASE_DIR, true, true, false, $baginfo);
	$bag->addFile($mbz, 'data/rad/datoteke/'.$storedfile->get_filename());
	$bag->addFile($xml_backup_file, 'data/rad/metapodaci/'.$backupfile.'.xml');
	$bag->addFile($xml_mods_file, 'data/rad/metapodaci/mods.xml');
	$bag->update();
	
	$bag->package(str_replace('.mbz', '.zip', $mbz), 'zip');
	$bagit_file = str_replace('.mbz', '.zip', $mbz);
}

$curl = new restClient(array('proxy' => true));
$curl->setAuth(get_config('local_dabar', "{$vu}_api_username"), get_config('local_dabar', "{$vu}_api_password"));
if($location){
	$ret = json_decode($curl->get($location));
}
else{
	$curl->post(get_config('local_dabar', "{$vu}_api_url"), array('name'=>'bagit', 'file'=>$bagit_file));
	if($curl->info['http_code'] == 202){
		$location = $curl->response['Location'];
		$ret = json_decode($curl->get($location));
	}
	else{
		$ret = new stdClass();
		$ret->poruka = $curl->info['http_code'];
		$ret->poruka = 'Unexpected HTTP code: '.$curl->info['http_code'];
	}
}
$url_params = array(
	'courseid' => $context->instanceid,
	'pathnamehash' => $pathnamehash,
	'contenthash' => $contenthash,
	'location' => $location,
	'stage' => 6,
	'url'=>new moodle_url('/local/dabar/upload/status.php')
);

$host = parse_url(get_config('local_dabar', "{$vu}_api_url"), PHP_URL_HOST);

$params = array(
	'kod' => ($ret->kod != 2 && $ret->kod != 3) ? true : false,
	'poruka' => $ret->poruka,
	'opis_greske' => $ret->opis_greske,
	'pid' => ($ret->kod == 1) ? "https://{$host}/islandora/object/{$ret->pid}" : false,
	'url_check'=>new moodle_url('/local/dabar/upload/status.php', $url_params),
	'courseid' => $context->instanceid,
	'pathnamehash' => $pathnamehash,
	'contenthash' => $contenthash,
	'location' => $location,
	'stage' => 6,
	'url'=>new moodle_url('/local/dabar/upload/status.php')
);
echo $OUTPUT->render_from_template('local_dabar/status', $params);
echo html_writer::end_tag('div');

echo $OUTPUT->footer();